@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -135,9 +136,9 @@ JSExtendedClass XPCNativeWrapper::sXPC_NW_JSClass = {
 // the wrapped native's flat JSObject, so the hook and args macro parameters
 // can be simply:
 //
-//      enumerate, (cx, obj)
+//      convert, (cx, obj, type, vp)
 //
-// in the call from XPC_NW_Enumerate, for example.
+// in the call from XPC_NW_Convert, for example.
 
 #define XPC_NW_CALL_HOOK(cx, obj, hook, args)                                 \
   return JS_GET_CLASS(cx, obj)->hook args;
@@ -153,8 +154,20 @@ ShouldBypassNativeWrapper(JSContext *cx, JSObject *obj)
   jsval flags;
 
   ::JS_GetReservedSlot(cx, obj, 0, &flags);
-  return !HAS_FLAGS(flags, FLAG_EXPLICIT) &&
-         !(::JS_GetTopScriptFilenameFlags(cx, nsnull) & JSFILENAME_SYSTEM);
+  if (HAS_FLAGS(flags, FLAG_EXPLICIT))
+    return JS_FALSE;
+
+  // Check what the script calling us looks like
+  JSScript *script = nsnull;
+  JSStackFrame *fp = cx->fp;
+  while(!script && fp) {
+    script = fp->script;
+    fp = fp->down;
+  }
+
+  // If there's no script, bypass for now because that's what the old code did.
+  // XXX FIXME: bug 341477 covers figuring out what we _should_ do.
+  return !script || !(::JS_GetScriptFilenameFlags(script) & JSFILENAME_SYSTEM);
 }
 
 #define XPC_NW_BYPASS_BASE(cx, obj, code)                                     \
@@ -308,6 +321,12 @@ RewrapIfDeepWrapper(JSContext *cx, JSObject *obj, jsval v, jsval *rval)
 #ifdef DEBUG_XPCNativeWrapper
       printf("Rewrapping for deep explicit wrapper\n");
 #endif
+      if (wrappedNative == XPCNativeWrapper::GetWrappedNative(cx, obj)) {
+        // Already wrapped, return the wrapper.
+        *rval = OBJECT_TO_JSVAL(obj);
+        return JS_TRUE;
+      }
+
       // |obj| is an explicit deep wrapper.  We want to construct another
       // explicit deep wrapper for |v|.  Just call XPCNativeWrapperCtor by hand
       // (passing null as the pre-created object it doesn't use anyway) so we
@@ -340,8 +359,15 @@ XPC_NW_FunctionWrapper(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                        jsval *rval)
 {
   JSObject *funObj = JSVAL_TO_OBJECT(argv[-2]);
-  if (!::JS_ObjectIsFunction(cx, funObj) ||
-      !XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
+  if (!::JS_ObjectIsFunction(cx, funObj)) {
+    obj = nsnull;
+  }
+
+  while (obj && !XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
+    obj = ::JS_GetPrototype(cx, obj);
+  }
+
+  if (!obj) {
     return ThrowException(NS_ERROR_UNEXPECTED, cx);
   }
 
@@ -380,10 +406,11 @@ XPC_NW_GetOrSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp,
     return JS_TRUE;
   }
 
-  // Be paranoid, don't let people use this as another object's
-  // prototype or anything like that.
-  if (!XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
-    return ThrowException(NS_ERROR_UNEXPECTED, cx);
+  while (!XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
+    obj = ::JS_GetPrototype(cx, obj);
+    if (!obj) {
+      return ThrowException(NS_ERROR_UNEXPECTED, cx);
+    }
   }
 
   XPCWrappedNative *wrappedNative =
@@ -589,9 +616,41 @@ XPC_NW_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_Enumerate(JSContext *cx, JSObject *obj)
 {
-  XPC_NW_BYPASS(cx, obj, enumerate, (cx, obj));
+  // We are being notified of a for-in loop or similar operation on this
+  // XPCNativeWrapper, so forward to the correct high-level object hook,
+  // OBJ_ENUMERATE on the XPCWrappedNative's object, called via the
+  // JS_Enumerate API.  Then reflect properties named by the enumerated
+  // identifiers from the wrapped native to the native wrapper.
 
-  return JS_TRUE;
+  XPCWrappedNative *wn = XPCNativeWrapper::GetWrappedNative(cx, obj);
+  if (!wn) {
+    return JS_TRUE;
+  }
+
+  JSIdArray *ida = JS_Enumerate(cx, wn->GetFlatJSObject());
+  if (!ida) {
+    return JS_FALSE;
+  }
+
+  JSBool ok = JS_TRUE;
+
+  for (jsint i = 0, n = ida->length; i < n; i++) {
+    JSObject *pobj;
+    JSProperty *prop;
+
+    // Let OBJ_LOOKUP_PROPERTY, in particular XPC_NW_NewResolve, figure
+    // out whether this id should be bypassed or reflected.
+    ok = OBJ_LOOKUP_PROPERTY(cx, obj, ida->vector[i], &pobj, &prop);
+    if (!ok) {
+      break;
+    }
+    if (prop) {
+      OBJ_DROP_PROPERTY(cx, pobj, prop);
+    }
+  }
+
+  JS_DestroyIdArray(cx, ida);
+  return ok;
 }
 
 static
@@ -657,10 +716,11 @@ XPC_NW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
     return JS_TRUE;
   }
 
-  // Be paranoid, don't let people use this as another object's
-  // prototype or anything like that.
-  if (!XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
-    return ThrowException(NS_ERROR_UNEXPECTED, cx);
+  while (!XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
+    obj = ::JS_GetPrototype(cx, obj);
+    if (!obj) {
+      return ThrowException(NS_ERROR_UNEXPECTED, cx);
+    }
   }
 
   XPCWrappedNative *wrappedNative =
@@ -692,7 +752,7 @@ XPC_NW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
       return JS_FALSE;
     }        
     
-    XPCWrappedNative* oldResolvingWrapper;
+    XPCWrappedNative* oldResolvingWrapper = nsnull;
     JSBool allowPropMods =
       NATIVE_HAS_FLAG(wrappedNative, AllowPropModsDuringResolve);
     if (allowPropMods) {
@@ -772,6 +832,9 @@ XPC_NW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
     return ThrowException(NS_ERROR_XPC_BAD_CONVERT_JS, cx);
   }
 
+  // Make sure memberval doesn't go away while we mess with it.
+  AUTO_MARK_JSVAL(ccx, memberval);
+  
   JSString *str = JSVAL_TO_STRING(id);
   if (!str) {
     return ThrowException(NS_ERROR_UNEXPECTED, cx);
@@ -801,6 +864,8 @@ XPC_NW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
       return JS_FALSE;
     }
 
+    AUTO_MARK_JSVAL(ccx, OBJECT_TO_JSVAL(funobj));
+
 #ifdef DEBUG_XPCNativeWrapper
     printf("Wrapping function object for %s\n",
            ::JS_GetStringBytes(JSVAL_TO_STRING(id)));
@@ -884,9 +949,42 @@ JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_Construct(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                  jsval *rval)
 {
+  // The object given to us by the JS engine is actually a stub object (the
+  // "new" object). This isn't any help to us, so instead use the function
+  // object of the constructor that we're calling (which is the native
+  // wrapper).
+  obj = JSVAL_TO_OBJECT(argv[-2]);
+
   XPC_NW_BYPASS_TEST(cx, obj, construct, (cx, obj, argc, argv, rval));
 
-  return JS_TRUE;
+  XPCWrappedNative *wrappedNative =
+    XPCNativeWrapper::GetWrappedNative(cx, obj);
+  if (!wrappedNative) {
+    return JS_TRUE;
+  }
+
+  JSBool retval = JS_TRUE;
+
+  if (!NATIVE_HAS_FLAG(wrappedNative, WantConstruct)) {
+    return ThrowException(NS_ERROR_INVALID_ARG, cx);
+  }
+
+  nsresult rv = wrappedNative->GetScriptableInfo()->
+    GetCallback()->Construct(wrappedNative, cx, obj, argc, argv, rval,
+                             &retval);
+  if (NS_FAILED(rv)) {
+    return ThrowException(rv, cx);
+  }
+
+  if (!retval) {
+    return JS_FALSE;
+  }
+
+  if (JSVAL_IS_PRIMITIVE(*rval)) {
+    return ThrowException(NS_ERROR_ILLEGAL_VALUE, cx);
+  }
+
+  return RewrapIfDeepWrapper(cx, obj, *rval, rval);
 }
 
 JS_STATIC_DLL_CALLBACK(JSBool)
@@ -960,6 +1058,15 @@ XPCNativeWrapperCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     if (!wrappedNative) {
       return ThrowException(NS_ERROR_INVALID_ARG, cx);
     }
+
+    // Prevent wrapping a double-wrapped JS object in an
+    // XPCNativeWrapper!
+    nsCOMPtr<nsIXPConnectWrappedJS> xpcwrappedjs =
+      do_QueryWrappedNative(wrappedNative);
+
+    if (xpcwrappedjs) {
+      return ThrowException(NS_ERROR_INVALID_ARG, cx);
+    }
   }
 
   JSObject *wrapperObj;
@@ -1127,18 +1234,15 @@ XPC_NW_Equality(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
   return JS_TRUE;
 }
 
-extern JSBool JS_DLL_CALLBACK
-XPC_WN_Shared_ToString(JSContext *cx, JSObject *obj,
-                       uintN argc, jsval *argv, jsval *vp);
-
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                 jsval *rval)
 {
-  // Be paranoid, don't let people use this as another object's
-  // prototype or anything like that.
-  if (!XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
-    return ThrowException(NS_ERROR_UNEXPECTED, cx);
+  while (!XPCNativeWrapper::IsNativeWrapper(cx, obj)) {
+    obj = ::JS_GetPrototype(cx, obj);
+    if (!obj) {
+      return ThrowException(NS_ERROR_UNEXPECTED, cx);
+    }
   }
 
   // Check whether toString was overridden in any object along
@@ -1148,6 +1252,10 @@ XPC_NW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     return JS_FALSE;
 
   jsid id = rt->GetStringID(XPCJSRuntime::IDX_TO_STRING);
+  jsval idAsVal;
+  if (!::JS_IdToValue(cx, id, &idAsVal)) {
+    return JS_FALSE;
+  }
 
   XPCWrappedNative *wrappedNative =
     XPCNativeWrapper::GetWrappedNative(cx, obj);
@@ -1164,49 +1272,71 @@ XPC_NW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     return JS_TRUE;
   }
 
+  // Someone is trying to call toString on our wrapped object.
   JSObject *wn_obj = wrappedNative->GetFlatJSObject();
+  XPCCallContext ccx(JS_CALLER, cx, wn_obj, nsnull, idAsVal);
+  if (!ccx.IsValid()) {
+    // Shouldn't really happen.
+    return ThrowException(NS_ERROR_FAILURE, cx);
+  }
+
+  XPCNativeInterface *iface = ccx.GetInterface();
+  XPCNativeMember *member = ccx.GetMember();
+  JSBool overridden = JS_FALSE;
   jsval toStringVal;
 
-  // Check whether toString has been overridden from its XPCWrappedNative
-  // default native method.
-  if (!OBJ_GET_PROPERTY(cx, wn_obj, id, &toStringVal)) {
-    return JS_FALSE;
+  // First, try to see if the object declares a toString in its IDL. If it does,
+  // then we need to defer to that.
+  if (iface && member) {
+    if (!member->GetValue(ccx, iface, &toStringVal)) {
+      return JS_FALSE;
+    }
+
+    overridden = member->IsMethod();
   }
 
-  JSBool overridden = JS_TypeOfValue(cx, toStringVal) != JSTYPE_FUNCTION;
-  if (!overridden) {
-    JSObject *toStringFunObj = JSVAL_TO_OBJECT(toStringVal);
-    JSFunction *toStringFun = (JSFunction*) ::JS_GetPrivate(cx, toStringFunObj);
-
-    overridden =
-      ::JS_GetFunctionNative(cx, toStringFun) != XPC_WN_Shared_ToString;
-  }
-
-  JSString* str;
+  JSString* str = nsnull;
   if (overridden) {
-    // Something overrides XPCWrappedNative.prototype.toString, we
-    // should defer to it.
+    // Defer to the IDL-declared toString.
 
-    str = ::JS_ValueToString(cx, OBJECT_TO_JSVAL(wn_obj));
-  } else {
+    AUTO_MARK_JSVAL(ccx, toStringVal);
+
+    JSObject *funobj = xpc_CloneJSFunction(ccx, JSVAL_TO_OBJECT(toStringVal),
+                                           wn_obj);
+    if (!funobj) {
+      return JS_FALSE;
+    }
+
+    jsval v;
+    if (!::JS_CallFunctionValue(cx, wn_obj, OBJECT_TO_JSVAL(funobj), argc, argv,
+                                &v)) {
+      return JS_FALSE;
+    }
+
+    if (JSVAL_IS_STRING(v)) {
+      str = JSVAL_TO_STRING(v);
+    }
+  }
+
+  if (!str) {
     // Ok, we do no damage, and add value, by returning our own idea
     // of what toString() should be.
+    // Note: We can't just call JS_ValueToString on the wrapped object. Instead,
+    // we need to call the wrapper's ToString in order to safely convert our
+    // object to a string.
 
     nsAutoString resultString;
     resultString.AppendLiteral("[object XPCNativeWrapper");
 
-    if (wrappedNative) {
-      JSString *str = ::JS_ValueToString(cx, OBJECT_TO_JSVAL(wn_obj));
-      if (!str) {
-        return JS_FALSE;
-      }
-
-      resultString.Append(' ');
-      resultString.Append(NS_REINTERPRET_CAST(PRUnichar *,
-                                              ::JS_GetStringChars(str)),
-                          ::JS_GetStringLength(str));
+    char *wrapperStr = wrappedNative->ToString(ccx);
+    if (!wrapperStr) {
+      return JS_FALSE;
     }
 
+    resultString.Append(' ');
+    resultString.AppendASCII(wrapperStr);
+    JS_smprintf_free(wrapperStr);
+
     resultString.Append(']');
 
     str = ::JS_NewUCStringCopyN(cx, NS_REINTERPRET_CAST(const jschar *,
@@ -1253,6 +1383,16 @@ XPCNativeWrapper::AttachNewConstructorObject(XPCCallContext &ccx,
 JSObject *
 XPCNativeWrapper::GetNewOrUsed(JSContext *cx, XPCWrappedNative *wrapper)
 {
+  // Prevent wrapping a double-wrapped JS object in an
+  // XPCNativeWrapper!
+  nsCOMPtr<nsIXPConnectWrappedJS> xpcwrappedjs(do_QueryWrappedNative(wrapper));
+
+  if (xpcwrappedjs) {
+    XPCThrower::Throw(NS_ERROR_INVALID_ARG, cx);
+
+    return nsnull;
+  }
+
   JSObject *obj = wrapper->GetNativeWrapper();
   if (obj) {
     return obj;
