@@ -510,9 +510,13 @@ nsXPConnect::InitClassesWithNewWrappedGlobal(JSContext * aJSContext,
         if(NS_FAILED(InitClasses(aJSContext, tempGlobal)))
             return UnexpectedFailure(NS_ERROR_FAILURE);
 
-        if(NS_FAILED(WrapNative(aJSContext, tempGlobal, aCOMObj, aIID,
-                                getter_AddRefs(holder))) || !holder)
-            return UnexpectedFailure(NS_ERROR_FAILURE);
+        nsresult rv;
+        if(!XPCConvert::NativeInterface2JSObject(ccx, getter_AddRefs(holder),
+                                                 aCOMObj, &aIID, tempGlobal,
+                                                 PR_FALSE, OBJ_IS_GLOBAL, &rv))
+            return UnexpectedFailure(rv);
+
+        NS_ASSERTION(NS_SUCCEEDED(rv) && holder, "Didn't wrap properly");
     }
 
     JSObject* globalJSObj;
@@ -586,9 +590,9 @@ nsXPConnect::WrapNative(JSContext * aJSContext,
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
     nsresult rv;
-    if(!XPCConvert::NativeInterface2JSObject(ccx, _retval,
-                                             aCOMObj, &aIID, aScope, PR_FALSE,
-                                             &rv))
+    if(!XPCConvert::NativeInterface2JSObject(ccx, _retval, aCOMObj, &aIID,
+                                             aScope, PR_FALSE,
+                                             OBJ_IS_NOT_GLOBAL, &rv))
         return rv;
 
 #ifdef DEBUG
@@ -848,13 +852,12 @@ nsXPConnect::ReparentScopeAwareWrappers(JSContext *aJSContext,
             // Now, reparent the wrapper, since we know that it wants to be
             // reparented.
 
-            XPCWrappedNative *junk;
+            nsRefPtr<XPCWrappedNative> junk;
             rv = XPCWrappedNative::ReparentWrapperIfFound(ccx, oldScope,
-                                                          newScope, aNewScope,
+                                                          newScope, newParent,
                                                           wrapper->GetIdentityObject(),
-                                                          &junk);
+                                                          getter_AddRefs(junk));
             NS_ENSURE_SUCCESS(rv, rv);
-            NS_IF_RELEASE(junk);
         }
     }
 
@@ -1199,6 +1202,57 @@ nsXPConnect::RestoreWrappedNativePrototype(JSContext * aJSContext,
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXPConnect::CreateSandbox(JSContext *cx, nsIPrincipal *principal,
+                           nsIXPConnectJSObjectHolder **_retval)
+{
+#ifdef XPCONNECT_STANDALONE
+    return NS_ERROR_NOT_AVAILABLE;
+#else /* XPCONNECT_STANDALONE */
+    XPCCallContext ccx(NATIVE_CALLER, cx);
+    if(!ccx.IsValid())
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    *_retval = nsnull;
+
+    jsval rval = JSVAL_VOID;
+    AUTO_MARK_JSVAL(ccx, &rval);
+
+    nsresult rv = xpc_CreateSandboxObject(cx, &rval, principal);
+    NS_ASSERTION(NS_FAILED(rv) || !JSVAL_IS_PRIMITIVE(rval),
+                 "Bad return value from xpc_CreateSandboxObject()!");
+
+    if (NS_SUCCEEDED(rv) && !JSVAL_IS_PRIMITIVE(rval)) {
+        *_retval = XPCJSObjectHolder::newHolder(cx, JSVAL_TO_OBJECT(rval));
+        NS_ENSURE_TRUE(*_retval, NS_ERROR_OUT_OF_MEMORY);
+
+        NS_ADDREF(*_retval);
+    }
+
+    return rv;
+#endif /* XPCONNECT_STANDALONE */
+}
+
+NS_IMETHODIMP
+nsXPConnect::EvalInSandboxObject(const nsAString& source, JSContext *cx,
+                                 nsIXPConnectJSObjectHolder *sandbox,
+                                 jsval *rval)
+{
+#ifdef XPCONNECT_STANDALONE
+    return NS_ERROR_NOT_AVAILABLE;
+#else /* XPCONNECT_STANDALONE */
+    if (!sandbox)
+        return NS_ERROR_INVALID_ARG;
+
+    JSObject *obj;
+    nsresult rv = sandbox->GetJSObject(&obj);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return xpc_EvalInSandbox(cx, obj, source,
+                             NS_ConvertUTF16toUTF8(source).get(), 1, rval);
+#endif /* XPCONNECT_STANDALONE */
+}
+
 /* nsIXPConnectJSObjectHolder getWrappedNativePrototype (in JSContextPtr aJSContext, in JSObjectPtr aScope, in nsIClassInfo aClassInfo); */
 NS_IMETHODIMP 
 nsXPConnect::GetWrappedNativePrototype(JSContext * aJSContext, 
@@ -1220,7 +1274,8 @@ nsXPConnect::GetWrappedNativePrototype(JSContext * aJSContext,
 
     AutoMarkingWrappedNativeProtoPtr proto(ccx);
     proto = XPCWrappedNativeProto::GetNewOrUsed(ccx, scope, aClassInfo, 
-                                                &sciProto, JS_FALSE);
+                                                &sciProto, JS_FALSE,
+                                                OBJ_IS_NOT_GLOBAL);
     if(!proto)
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
