@@ -68,6 +68,12 @@
 #include "nsCSSFrameConstructor.h"
 #include "nsGUIEvent.h"
 #include "nsIRootBox.h"
+#include "nsIFocusController.h"
+#include "nsIDocShellTreeItem.h"
+#include "nsIDocShell.h"
+#include "nsPIDOMWindow.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsIBaseWindow.h"
 
 #define NS_MENU_POPUP_LIST_INDEX   0
 
@@ -342,6 +348,9 @@ nsPopupSetFrame::ShowPopup(nsIContent* aElementContent, nsIContent* aPopupConten
                            const nsString& aPopupType, const nsString& anAnchorAlignment,
                            const nsString& aPopupAlignment)
 {
+  if (!MayOpenPopup(this))
+    return NS_OK;
+
   nsWeakFrame weakFrame(this);
   // First fire the popupshowing event.
   if (!OnCreate(aXPos, aYPos, aPopupContent) || !weakFrame.IsAlive())
@@ -513,6 +522,7 @@ nsPopupSetFrame::OpenPopup(nsPopupFrameList* aEntry, PRBool aActivateFlag)
   nsWeakFrame weakFrame(this);
   nsIFrame* activeChild = aEntry->mPopupFrame;
   nsWeakFrame weakPopupFrame(activeChild);
+  nsRefPtr<nsPresContext> presContext = GetPresContext();
   nsCOMPtr<nsIContent> popupContent = aEntry->mPopupContent;
   PRBool createHandlerSucceeded = aEntry->mCreateHandlerSucceeded;
   nsAutoString popupType = aEntry->mPopupType;
@@ -539,7 +549,7 @@ nsPopupSetFrame::OpenPopup(nsPopupFrameList* aEntry, PRBool aActivateFlag)
     }
   }
   else {
-    if (createHandlerSucceeded && !OnDestroy(aEntry->mPopupContent))
+    if (createHandlerSucceeded && !OnDestroy(popupContent))
       return;
 
     // Unregister, but not if we're a tooltip
@@ -555,11 +565,10 @@ nsPopupSetFrame::OpenPopup(nsPopupFrameList* aEntry, PRBool aActivateFlag)
     if (childPopup)
       childPopup->RemoveKeyboardNavigator();
 
-    nsRefPtr<nsPresContext> presContext = mPresContext;
-    nsCOMPtr<nsIContent> content = aEntry->mPopupContent;
-    ActivatePopup(aEntry, PR_FALSE);
+    if (weakPopupFrame.IsAlive())
+      ActivatePopup(aEntry, PR_FALSE);
 
-    OnDestroyed(presContext, content);
+    OnDestroyed(presContext, popupContent);
   }
 
   if (weakFrame.IsAlive()) {
@@ -626,11 +635,10 @@ nsPopupSetFrame::OnCreate(PRInt32 aX, PRInt32 aY, nsIContent* aPopupContent)
 
   if (aPopupContent) {
     nsCOMPtr<nsIContent> kungFuDeathGrip(aPopupContent);
-    nsIPresShell *shell = mPresContext->GetPresShell();
+    nsCOMPtr<nsIPresShell> shell = mPresContext->GetPresShell();
     if (shell) {
       nsresult rv = shell->HandleDOMEventWithTarget(aPopupContent, &event,
                                                     &status);
-      // shell may no longer be alive, don't use it here unless you keep a ref
       if ( NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault )
         return PR_FALSE;
     }
@@ -704,11 +712,10 @@ nsPopupSetFrame::OnCreated(PRInt32 aX, PRInt32 aY, nsIContent* aPopupContent)
   event.point.y = aY;
 
   if (aPopupContent) {
-    nsIPresShell *shell = mPresContext->GetPresShell();
+    nsCOMPtr<nsIPresShell> shell = mPresContext->GetPresShell();
     if (shell) {
       nsresult rv = shell->HandleDOMEventWithTarget(aPopupContent, &event,
                                                     &status);
-      // shell may no longer be alive, don't use it here unless you keep a ref
       if ( NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault )
         return PR_FALSE;
     }
@@ -725,11 +732,10 @@ nsPopupSetFrame::OnDestroy(nsIContent* aPopupContent)
                      nsMouseEvent::eReal);
 
   if (aPopupContent) {
-    nsIPresShell *shell = mPresContext->GetPresShell();
+    nsCOMPtr<nsIPresShell> shell = mPresContext->GetPresShell();
     if (shell) {
       nsresult rv = shell->HandleDOMEventWithTarget(aPopupContent, &event,
                                                     &status);
-      // shell may no longer be alive, don't use it here unless you keep a ref
       if ( NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault )
         return PR_FALSE;
     }
@@ -746,11 +752,10 @@ nsPopupSetFrame::OnDestroyed(nsPresContext* aPresContext,
                      nsMouseEvent::eReal);
 
   if (aPopupContent && aPresContext) {
-    nsIPresShell *shell = aPresContext->GetPresShell();
+    nsCOMPtr<nsIPresShell> shell = aPresContext->GetPresShell();
     if (shell) {
       nsresult rv = shell->HandleDOMEventWithTarget(aPopupContent, &event,
                                                     &status);
-      // shell may no longer be alive, don't use it here unless you keep a ref
       if ( NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault )
         return PR_FALSE;
     }
@@ -832,3 +837,42 @@ nsPopupSetFrame::AddPopupFrame(nsIFrame* aPopup)
   return NS_OK;
 }
 
+//static
+PRBool
+nsPopupSetFrame::MayOpenPopup(nsIFrame* aFrame)
+{
+  nsCOMPtr<nsISupports> cont = aFrame->GetPresContext()->GetContainer();
+  nsCOMPtr<nsIDocShellTreeItem> dsti = do_QueryInterface(cont);
+  if (!dsti)
+    return PR_FALSE;
+
+  // chrome shells can always open popups
+  PRInt32 type = -1;
+  if (NS_SUCCEEDED(dsti->GetItemType(&type)) && type == nsIDocShellTreeItem::typeChrome)
+    return PR_TRUE;
+
+  nsCOMPtr<nsIDocShell> shell = do_QueryInterface(dsti);
+  if (!shell)
+    return PR_FALSE;
+
+  nsCOMPtr<nsPIDOMWindow> win = do_GetInterface(shell);
+  if (!win)
+    return PR_FALSE;
+
+  // only allow popups in active windows
+  PRBool active;
+  nsIFocusController* focusController = win->GetRootFocusController();
+  focusController->GetActive(&active);
+  if (!active)
+    return PR_FALSE;
+
+  nsCOMPtr<nsIBaseWindow> baseWin = do_QueryInterface(shell);
+  if (!baseWin)
+    return PR_FALSE;
+
+  // only allow popups in visible frames
+  PRBool visible;
+  baseWin->GetVisibility(&visible);
+  return visible;
+}
+
