@@ -101,10 +101,6 @@ var helpFileDS;
 # reduction on all links within the current help set.
 var helpBaseURI;
 
-var gIgnoreFocus = false;
-var gClickSelectsAll;
-var gIgnoreClick = false;
-
 /* defaultTopic is either set
    1. in the openHelp() call, passed as an argument to the Help window and
       evaluated in init(), or
@@ -144,7 +140,7 @@ function init() {
   strBundle = document.getElementById("bundle_help");
   emptySearchText = strBundle.getString("emptySearchText");
 
-  initFindBar();
+  gFindBar.initFindBar();
 
   // Get the content pack, base URL, and help topic
   var helpTopic = defaultTopic;
@@ -179,8 +175,9 @@ function init() {
 
   webProgress.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
 
-  gClickSelectsAll = getBoolPref("browser.urlbar.clickSelectsAll", true);
-  
+  var searchBox = document.getElementById("findText");
+  searchBox.clickSelectsAll = getBoolPref("browser.urlbar.clickSelectsAll", true);
+
   setTimeout(focusSearch, 0);
 }
 
@@ -239,23 +236,24 @@ function loadHelpRDF() {
       var iterator = RDFContainer.GetElements();
       while (iterator.hasMoreElements()) {
         var panelDef = iterator.getNext();
-        var panelID = getAttribute(helpFileDS, panelDef, NC_PANELID, null);
 
-        var datasources = getAttribute(helpFileDS, panelDef, NC_DATASOURCES, "");
-        // if datasources is "", there's nothing to load
+        var panelID        = getAttribute(helpFileDS, panelDef, NC_PANELID, null);
+        var datasources    = getAttribute(helpFileDS, panelDef, NC_DATASOURCES, "");
+        var panelPlatforms = getAttribute(helpFileDS, panelDef, NC_PLATFORM, platform);
+
+        if (panelPlatforms.split(/\s+/).indexOf(platform) == -1)
+          continue; // ignore datasources for other platforms
+
+        // empty datasources are valid on search panel definitions
+        // convert them to "rdf:null" which can be filtered and ignored
         if (!datasources)
-          continue;
+          datasources = "rdf:null";
 
         datasources = normalizeLinks(helpBaseURI, datasources);
 
-        var panelPlatforms = getAttribute(helpFileDS, panelDef, NC_PLATFORM, platform);
-        panelPlatforms = panelPlatforms.split(/\s+/);
-
-        if (panelPlatforms.indexOf(platform) == -1)
-          continue; // ignore datasources for other platforms
-
         var datasourceArray = datasources.split(/\s+/)
-                                          .map(RDF.GetDataSourceBlocking);
+                                         .filter(function(x) { return x != "rdf:null"; })
+                                         .map(RDF.GetDataSourceBlocking);
 
         // Cache Additional Datasources to Augment Search Datasources.
         if (panelID == "search") {
@@ -333,7 +331,7 @@ function filterSeqByPlatform(aDatasource, aNode, aCurrentLevel) {
                                      NC_PLATFORM,
                                      platform);
 
-    if (nodePlatforms.indexOf(platform) == -1) { // node is for another platform
+    if (nodePlatforms.split(/\s+/).indexOf(platform) == -1) { // node is for another platform
       var currentNode = currentTarget.QueryInterface(Components.interfaces.nsIRDFNode);
       // "false" because we don't want to renumber elements in the container
       RDFC.RemoveElement(currentNode, false);
@@ -378,7 +376,8 @@ function getLink(ID) {
 
     // URIs include both the ID part and the base file name,
     // so we need to check for a matching ID in each datasource
-    var tocDSArray = gTocDSList.split(/\s+/);
+    var tocDSArray = gTocDSList.split(/\s+/)
+                               .filter(function(x) { return x != "rdf:null"; });
 
     for (var i = 0; i < tocDSArray.length; i++) {
       var resource = RDF.GetResource(tocDSArray[i] + "#" + ID);
@@ -426,33 +425,6 @@ function goBack() {
   }
 }
 
-/* copied from browser.js */
-function BrowserReloadWithFlags(reloadFlags) {
-    /* First, we'll try to use the session history object to reload so 
-     * that framesets are handled properly. If we're in a special 
-     * window (such as view-source) that has no session history, fall 
-     * back on using the web navigation's reload method.
-     */
-
-    var webNav = getWebNavigation();
-    try {
-      var sh = webNav.sessionHistory;
-      if (sh)
-        webNav = sh.QueryInterface(Components.interfaces.nsIWebNavigation);
-    } catch (e) {
-    }
-
-    try {
-      webNav.reload(reloadFlags);
-    } catch (e) {
-    }
-}
-
-function reload() {
-  const reloadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
-  return BrowserReloadWithFlags(reloadFlags);
-}
-
 function goForward() {
     try
     {
@@ -630,33 +602,6 @@ function onselect_loadURI(tree) {
     }// when switching between tabs a spurious row number is returned.
 }
 
-// copied from browser.js to handle the searchbar
-function SearchbarFocusHandler(aEvent, aElt)
-{
-  if (gIgnoreFocus)
-    gIgnoreFocus = false;
-  else if (gClickSelectsAll)
-    aElt.select();
-}
-
-function SearchbarMouseDownHandler(aEvent, aElt)
-{
-  if (aElt.hasAttribute("focused")) {
-    gIgnoreClick = true;
-  } else {
-    gIgnoreFocus = true;
-    gIgnoreClick = false;
-    aElt.setSelectionRange(0, 0);
-  }
-}
-
-function SearchbarClickHandler(aEvent, aElt)
-{
-  if (!gIgnoreClick && gClickSelectsAll && aElt.selectionStart == aElt.selectionEnd)
-    aElt.select();
-}
-
-
 function focusSearch() {
   var searchBox = document.getElementById("findText");
   searchBox.focus();
@@ -813,7 +758,7 @@ function getLiteralValue(literal, defaultValue) {
     return null;
 }
 
-# Write debug string to javascript console.
+# Write debug string to error console.
 function log(aText) {
     CONSOLE_SERVICE.logStringMessage(aText);
 }
